
	cpu 286
	org 0x7c00
	bits 16

	;; Reset registers
	mov ax, 0
	mov ds, ax
	mov es, ax
	mov ss, ax

	;; put the stack right below where the OS is loaded
	;; delightfully devilish, seymour
	mov sp, 0x7c00				

	;; Set up variables
	%include "variables.asm"
	
	;; Active display page 0
	mov ah, 0x05
	mov al, 0x00
	int 0x10

	;; 80x25, 16 colours
	mov ah, 0
	mov al, 0x03
	int 0x10

	mov bx, STARTUP
	call print

	;; testing
	;///mov ax, 0x300
	;//call print_hex

	;; Load sectors 2,3,4,5(more OS)
	mov cx, 2					;sector & cylinder
	mov dh, 0					;head
	mov al, 4					;Num sectors
	mov bx, 0x7E00				;RAM
	call read_sector

	mov al, '>'					;cmd line
	call print_char
	mov al, ' '
	call print_char
	
loop:
	mov ah, 0
	int 0x16					;Reads a character to al
	cmp al, 0x8				;Backspace
	je kb_bs
	cmp al, 0xd
	je kb_ent

	;;Store character in CMD_BUFFER
	mov bx, CMD_BUFFER
	add bx, [CMD_BUFFER_POS]
	mov [bx], al
	mov ah, [CMD_BUFFER_POS]
	add ah, 1
	mov [CMD_BUFFER_POS], ah
	
	mov ah, 0x0e
	int 0x10
	
	jmp loop

	;; Print has to be kept here so that disk errors can be printed
	%include "print.asm"
	%include "disk.asm"
	
;//CMD_BUFFER:	times 64 db 0 		;63 char buffer for command
CMD_BUFFER_POS:	dw 0
CURSOR_H:	db 0				;Not used for now
CURSOR_W:	db 0
	
STARTUP:	db "Starting Steve16...", 0xA, 0xD,
	db "READY.", 0xA, 0xD, 0

	times 510-($-$$) db 0
	dw 0xaa55

	;; SECTOR 2 & 3 & 4
	%include "string.asm"
	%include "textedit.asm"
	%include "memtest.asm"
	%include "snake.asm"

kb_bs:
	cmp [CMD_BUFFER_POS], byte 0 ;Ignore the backspace if CMD_BUFFER is empty
	je loop

	;; If the cursor is at the start of the line, move it to the line above it, far right
	mov bh, 0
	mov ah, 0x03
	int 0x10					;Get the cursor's position

	cmp dl, 0 					;In the first column
	jne kb_skip_upline

	
	mov ah, 0x02
	mov dl, 79					;80 column display
	sub dh, 1					;One row up
	int 0x10

	mov ah, 0x0E
	mov al, ' '
	int 0x10 					;Print a space, putting us back where we were before

	mov ah, 0x02
	int 0x10					;And back up we go

	jmp kb_bs_done
	
kb_skip_upline:	
	;; Backspace, space, backspace to overwrite character
	mov ah, 0x0e
	int 0x10
	mov al, ' '
	int 0x10
	mov al, 0x08
	int 0x10

kb_bs_done:

	;; Decrement buffer count
	mov bx, [CMD_BUFFER_POS]	;Could combine this with the cmp operation above to save an instruction
	;//cmp bx, 0				;Not needed - check is performed above
	;//je loop
	sub bx, 1
	mov [CMD_BUFFER_POS], bx

	jmp loop

kb_ent:
	;; move cursor down 1
	mov ah, 0x0E
	mov al, 0x0D
	int 0x10
	mov al, 0xA
	int 0x10
	
	;; Process command
	call cmd

	;; Print >
	mov al, '>'
	call print_char
	mov al, ' '
	call print_char
	
	jmp loop

cmd:
	;; Add a null terminator in the right spot
	mov bx, CMD_BUFFER
	add bx, [CMD_BUFFER_POS]
	mov [bx], byte 0
	;//mov bx, CMD_BUFFER
	;//call print
	mov cx, CMD_BUFFER
	mov dx, CMD_HELP_STR
	call strcmp
	cmp cl, 1
	je cmd_help

	mov dx, CMD_SEDIT_STR
	call strcmp
	cmp cl, 1
	je cmd_sedit

	mov dx, CMD_CLEAR_STR
	call strcmp
	cmp cl, 1
	je cmd_clear

	mov dx, CMD_MEMTEST_STR
	call strcmp
	cmp cl, 1
	je cmd_memtest

	mov dx, CMD_STRSPLIT_STR
	call strcmp
	cmp cl, 1
	je cmd_strsplit

	mov dx, CMD_SNAKE_STR
	call strcmp
	cmp cl, 1
	je cmd_snake

	;; don't print unknown command message if cmd is empty
	cmp [CMD_BUFFER_POS], byte 0
	je clear_cmd_buffer
	

	mov bx, RTN_UNKNOWN_STR
	call print

clear_cmd_buffer:	
	mov [CMD_BUFFER_POS], byte 0
	ret
	
cmd_memtest:
	pusha
	mov bx, 0x8B00
	mov ax, 0xFFFF
	call memtest_init
	mov al, 'D'					;done
	call print_char
	popa
	jmp clear_cmd_buffer

cmd_help:
	mov bx, RTN_HELP_STR
	call print
	jmp clear_cmd_buffer

cmd_sedit:
	call sedit_init
	jmp clear_cmd_buffer

cmd_clear:
	mov ax, 0x03
	int 0x10
	jmp clear_cmd_buffer

cmd_strsplit:
	mov bx, STRSPLIT_TEST_STR
	call print					;print starting string
	
	mov al, ' '
	mov dx, STRSPLIT_TEST_OUTPUT
	call strsplit

	;; print out the new strings
	mov bx, STRSPLIT_TEST_OUTPUT
	mov al, ','
	call print_array
	
	jmp clear_cmd_buffer

cmd_snake:
	call snake_init
	jmp clear_cmd_buffer
	
CMD_MEMTEST_STR:	 db "MEMTEST", 0
CMD_SEDIT_STR: db "SEDIT", 0
CMD_HELP_STR:  db "HELP", 0
CMD_CLEAR_STR: db "CLEAR", 0
CMD_STRSPLIT_STR:	db "STRSP", 0
CMD_SNAKE_STR:	db "SNAKE", 0

STRSPLIT_TEST_STR:	db "this is a test string", 0
STRSPLIT_TEST_OUTPUT:	times 16 dw 0 ;16 element 2-byte array
	
RTN_HELP_STR:
	db "Steve16 v0.1", 0xA, 0xD
	db "Available commands:", 0xA, 0xD
	db "    HELP        This screen", 0xA, 0xD
	db "    SEDIT       Steve16 Text Editor", 0xA, 0xD
	db "    CLEAR       Clears the screen", 0xA, 0xD
	db "    MEMTEST     Runs a memory test", 0xA, 0xD
	db "    SNAKE       Play a game of snake", 0xA, 0xD
	db 0

RTN_UNKNOWN_STR:
	db "Unknown command. Type HELP for help.", 0xA, 0xD, 0

	times (2048+512)-($-$$) db 0

	;; Everything after this is not loaded
	;; The padding is so we can write there in the future
	times(0x18000) db 0
